@@ -3,6 +3,7 @@
  * (c) 2006 Thierry MERLE <thierry.merle@free.fr>
  * (c) 2009, 2010 Louis Lagendijk
  * (c) 2014, 2022 Markus Heinz
+ * (c) 2024 Alexander Toresson, Markus Heinz
  *
  * Code taken from epson.c. Commands taken from CanonUtil 0.07 
  * (http://xwtools.automatix.de/)
@@ -302,6 +303,7 @@ int get_ink_level_canon(const int port, const char* device_file,
   int retry = 6; /* You can change this, but keep same parity */
   levelTab lt;
   int xmlFound = 0;
+  int result;
 
   if ((port == BJNP) || (port == CUSTOM_BJNP)) {
     bjnp_get_printer_status(port, device_file,  portnumber, buffer);
@@ -413,7 +415,7 @@ int get_ink_level_canon(const int port, const char* device_file,
 
       }
 
-    } while (!indexDOC && !indexDWS && !indexCHD && !indexCHD && --retry);
+    } while (!indexDOC && !indexDWS && !indexCHD && !indexCIR && --retry);
   }
 
   if (xmlFound) {
@@ -422,10 +424,13 @@ int get_ink_level_canon(const int port, const char* device_file,
     printf("XML response detected\n");
 #endif
 
-    return parse_level_from_canon_xml((unsigned char *) buffer, level);
+    result = parse_level_from_canon_xml((unsigned char *) buffer, level);
+    if (level->status == RESPONSE_VALID) {
+      return result;
+    }
   }
 
-  if (!indexDOC && !indexDWS && !indexCHD && !indexCHD) {
+  if (!indexDOC && !indexDWS && !indexCHD && !indexCIR) {
 	  
 #ifdef DEBUG
 
