@@ -1,5 +1,6 @@
 /* d4lib.c 
  * Copyright (C) 2001 Jean-Jacques Sarton jj.sarton@t-online.de
+ * Copyright (C) 2014, 2015 Markus Heinz
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,13 +26,13 @@
  * This may be reached if this file is compiled with the TEST
  * option set.
  *
- * I dont own a Stylus Scanner and I am not able to test this
- * code, as desired. Printing on a Stylus Photo 1290 work fine
+ * I don't own a Stylus Scanner and I am not able to test this
+ * code, as desired. Printing on a Stylus Photo 1290 works fine
  * with this.
  *
  * The best way to get the Stylus Scanner working is to test
- * this and also correct my possibly errors.
- * Programming knowledge will be helpfull for this.
+ * this and also correct my possible errors.
+ * Programming knowledge will be helpful for this.
  *
  */
  
@@ -51,7 +52,7 @@
 #include <ctype.h>
 
 #include "d4lib.h"
-
+#include "printer-io.h"
 
 #ifndef RDTIMEOUT
 #define RDTIMEOUT 2000
@@ -71,7 +72,7 @@ int debugD4     = 0;
 typedef void (*signalHandler_t)(int);
 static signalHandler_t sig;
 static int timeoutGot = 0;
-static int _readData(int fd, unsigned char *buf, int len);
+static int _readData(printer_descriptor *desc, unsigned char *buf, int len);
 
 /* commands for the D4 protocol
 
@@ -145,28 +146,28 @@ typedef struct errorMessage_s
 
 static errorMessage_t errorMessage[] =
 {
-   { 0x01, "Unable to begin conversation, try later."              ,0 },
-   { 0x02, "Protocol revision not supported."                      ,1 },
-   { 0x03, "Transaction channel cant be closed."                  ,0 },
-   { 0x04, "No sufficient resources available now."                ,0 },
-   { 0x05, "Connection denied."                                    ,1 },
-   { 0x06, "Channel allready open."                                ,0 },
-   { 0x07, "Credit overflow, previous credit remain valid."        ,0 },
-   { 0x08, "Channel is not open."                                  ,1 },
-   { 0x09, "Service not available on specified socket."            ,1 },
-   { 0x0a, "Service name to socket ID failed."                     ,1 },
-   { 0x0b, "Init transaction failed."                              ,1 },
-   { 0x0c, "Invalid packet size."                                  ,1 },
-   { 0x0d, "Requested packed size is 0, no data can be transfered.",0 },
-   { 0x80, "Malformed packet, ignored."                            ,1 },
-   { 0x81, "No credit for received packet, ignored"                ,0 },
-   { 0x82, "Reply dont match with outstanding command, ignored."  ,1 },
-   { 0x83, "Packet size greater as negotiated size."               ,1 },
-   { 0x84, "Data received for a non opened channel."               ,1 },
-   { 0x85, "Reply with unknown result value received."             ,1 },
-   { 0x86, "Piggybacked credit in data packet cause overflow."     ,1 },
-   { 0x87, "Unknown 1284.4 Reply."                                 ,0 },
-   { 0x00, NULL                                                    ,0 }
+   { 0x01, "Unable to begin conversation, try later."                 ,0 },
+   { 0x02, "Protocol revision not supported."                         ,1 },
+   { 0x03, "Transaction channel can't be closed."                     ,0 },
+   { 0x04, "No sufficient resources available now."                   ,0 },
+   { 0x05, "Connection denied."                                       ,1 },
+   { 0x06, "Channel already open."                                    ,0 },
+   { 0x07, "Credit overflow, previous credit remain valid."           ,0 },
+   { 0x08, "Channel is not open."                                     ,1 },
+   { 0x09, "Service not available on specified socket."               ,1 },
+   { 0x0a, "Service name to socket ID failed."                        ,1 },
+   { 0x0b, "Init transaction failed."                                 ,1 },
+   { 0x0c, "Invalid packet size."                                     ,1 },
+   { 0x0d, "Requested packed size is 0, no data can be transferred."  ,0 },
+   { 0x80, "Malformed packet, ignored."                               ,1 },
+   { 0x81, "No credit for received packet, ignored"                   ,0 },
+   { 0x82, "Reply doesn't match with outstanding command, ignored."   ,1 },
+   { 0x83, "Packet size greater than negotiated size."                ,1 },
+   { 0x84, "Data received for a non opened channel."                  ,1 },
+   { 0x85, "Reply with unknown result value received."                ,1 },
+   { 0x86, "Piggybacked credit in data packet caused overflow."       ,1 },
+   { 0x87, "Unknown 1284.4 Reply."                                    ,0 },
+   { 0x00, NULL                                                       ,0 }
 };
 
 #define RESET_TIMER(ti,oti) { signal(SIGALRM, sig); \
@@ -259,15 +260,16 @@ static void printHexValues(const char *dir, const unsigned char *buf, int len)
      }
 }
 
-int SafeWrite(int fd, const void *data, int len)
+int SafeWrite(struct printer_descriptor *desc, void *data, int len)
 {
-  int status;
-  int retries=30;
+  int status = 0;
+  int retries = 30;
   if (debugD4)
     printHexValues("SafeWrite: ", data, len);
   do
     {
-      status = write(fd, data, len);
+      write_wrapper(desc, (char *) data, len, &status);
+
       if(status < len)
 	usleep(d4WrTimeout);
       retries--;
@@ -367,7 +369,7 @@ static void printCmdType(unsigned char *cmd)
 /*                                                                 */
 /*******************************************************************/
 
-static int writeCmd(int fd, unsigned char *cmd, int len)
+static int writeCmd(printer_descriptor *desc, unsigned char *cmd, int len)
 {
    int w;
    int i = 0;
@@ -401,7 +403,7 @@ static int writeCmd(int fd, unsigned char *cmd, int len)
    while ( i < len )
    {
       SET_TIMER(ti,oti,d4WrTimeout);         
-      w = SafeWrite(fd, cmd+i,len-i);
+      w = SafeWrite(desc, cmd+i,len-i);
       RESET_TIMER(ti,oti);
       if ( w < 0 )
       {
@@ -442,7 +444,7 @@ static int writeCmd(int fd, unsigned char *cmd, int len)
 /*                                                                 */
 /*******************************************************************/
 
-int readAnswer(int fd, unsigned char *buf, int len)
+int readAnswer(printer_descriptor *desc,  unsigned char *buf, int len)
 {
    int rd    = 0;
    int total = 0;
@@ -457,7 +459,7 @@ int readAnswer(int fd, unsigned char *buf, int len)
    /* for error handling in case of timeout */
    timeoutGot = 0;
 
-   /* set errno to 0 in order to get correct informations */
+   /* set errno to 0 in order to get correct information */
    /* in case of error                                    */
    errno = 0;
 
@@ -468,7 +470,11 @@ int readAnswer(int fd, unsigned char *buf, int len)
    while ( total < len )
    {
       SET_TIMER(ti,oti, d4RdTimeout);
-      rd = read(fd, buf+total, len-total);
+
+      if (read_wrapper(desc, (char *) buf, len, &rd, 0) != PRINTER_SUCCESS) {
+	return -1;
+      }
+
       if (debugD4)
 	{
 	  if (first_read)
@@ -543,7 +549,7 @@ int readAnswer(int fd, unsigned char *buf, int len)
    return total;
 }
 
-static void _flushData(int fd)
+static void _flushData(printer_descriptor *desc)
 {
    int rd    = 0;
    struct itimerval ti, oti;
@@ -555,7 +561,7 @@ static void _flushData(int fd)
    /* for error handling in case of timeout */
    timeoutGot = 0;
 
-   /* set errno to 0 in order to get correct informations */
+   /* set errno to 0 in order to get correct information */
    /* in case of error                                    */
    errno = 0;
 
@@ -565,13 +571,14 @@ static void _flushData(int fd)
      {
        usleep(d4RdTimeout);
        SET_TIMER(ti,oti, d4RdTimeout);
-       rd = read(fd, buf, len);
+
+       read_wrapper(desc, buf, len, &rd, 0);
+
        if (debugD4)
-	 fprintf(stderr, "flush: read: %i %s\n", rd,
-		 rd < 0 && errno != 0 ?strerror(errno) : "");
+	 fprintf(stderr, "flush: read: %i\n", rd);
        RESET_TIMER(ti,oti);
        count--;
-     } while ( count > 0 && (rd > 0 || (rd < 0 && errno == EAGAIN)));
+     } while ( count > 0 && rd > 0 );
 }
 
 /*******************************************************************/
@@ -585,12 +592,22 @@ static void _flushData(int fd)
 /*                                                                 */
 /*******************************************************************/
 
-static int _readData(int fd, unsigned char *buf, int len)
+/*
+ * TODO: Reading six bytes first, then calculating the length and reading
+ * the rest does not seems to work with libusb. An overflow occurs.
+ *
+ * It should be tried to read all data in one read and then
+ * checking if everything is there.
+ */
+
+static int _readData(printer_descriptor *desc, unsigned char *buf, int len)
 {
    int rd    = 0;
    int total = 0;
    int toGet = 0;
-   unsigned char  header[6];
+   int tries = 0;
+   unsigned char header[6];
+   unsigned char buffer[1024];
    struct timeval beg, end;
    long dt;
    struct itimerval ti, oti;
@@ -598,74 +615,147 @@ static int _readData(int fd, unsigned char *buf, int len)
    /* set errno to 0 in order to get correct informations */
    /* in case of error                                    */
    errno = 0;
+   
+   /* It should be checked if this implementation works for device node
+    *  communication too.
+    */
+   if (desc->mode == PRINTER_MODE_LIBUSB) {
 
-   /* read the first 6 bytes */
-   gettimeofday(&beg, NULL);
-   while ( total < 6 )
-   {
-      SET_TIMER(ti,oti, d4RdTimeout);
-      rd = read(fd, header+total, 6-total);
-      RESET_TIMER(ti,oti);
-      if ( rd <= 0 )
-      {
-         gettimeofday(&end, NULL);
-         dt  = (end.tv_sec  - beg.tv_sec) * 1000;
-         dt += (end.tv_usec - beg.tv_usec) / 1000;
-         if ( dt > d4RdTimeout*3 )
-         {
-            if ( debugD4 )
-               fprintf(stderr,"Timeout at _readData(), dt = %ld ms\n", dt);
-            return -1;
-            break;
-         }
-         continue;
-      }
-      else
-      {
-         total += rd;
-      }
-   }
+     /* There is no timer because libusb has a timeout by itself */
 
-   if ( debugD4 )
-      printHexValues("Recv: ",header,total);
+     do {
 
-   if ( total == 6 )
-   {
-      toGet = (header[2] >> 8) + header[3] - 6;
-      if (debugD4)
-	fprintf(stderr, "toGet: %i\n", toGet);
-      total = 0;
-      gettimeofday(&beg, NULL);
-      while ( total < toGet )
-      {
-         SET_TIMER(ti,oti, d4RdTimeout);
-         rd = read(fd, buf+total, toGet-total);
-         RESET_TIMER(ti,oti);
-         if ( rd <= 0 )
-         {
-            gettimeofday(&end, NULL);
-            dt  = (end.tv_sec  - beg.tv_sec) * 1000;
-            dt += (end.tv_usec - beg.tv_usec) / 1000;
-            if ( dt > d4RdTimeout*3 )
-            {
-               if ( debugD4 )
-                  fprintf(stderr,"Timeout at _readData(), dt = %ld ms\n",dt);
-               return -1;
-               break;
-            }
-            continue;
+       if (read_wrapper(desc, (char *) buffer, sizeof(buffer), &rd, 0) 
+	   != PRINTER_SUCCESS) {
+	 return -1;
+       }
+
+       tries++;
+
+#ifdef DEBUG
+       printf("Tries to read so far: %d\n", tries);
+#endif
+
+     } while (rd == 0); // Trying forever. Maybe a limit is needed?
+
+     if (rd >= 6) {
+       toGet = (buffer[2] >> 8) + buffer[3] - 6;
+
+#ifdef DEBUG
+       printf("%d bytes to get\n", toGet);
+#endif
+
+       if (toGet <= len) {
+	 memcpy (buf, buffer + 6, toGet);
+       } else {
+	 
+#ifdef DEBUG
+	 printf("Destination buffer too small: %d, needed %d\n", len, toGet);
+#endif
+
+	 return -1;
+       }
+
+       if ( debugD4 ) {
+         printHexValues("Recv: ",buf, toGet);
+       }
+
+       return toGet;
+
+     } else {
+
+#ifdef DEBUG
+       printf("Insufficient data read: %d, need at least 6\n", rd);
+#endif
+
+       return -1;
+     }
+
+   } else {
+
+     /* read the first 6 bytes */
+     gettimeofday(&beg, NULL);
+     while ( total < 6 ) {
+       SET_TIMER(ti,oti, d4RdTimeout);
+
+       if (read_wrapper(desc, (char *) header + total, 6 - total, &rd, 0) 
+	   != PRINTER_SUCCESS) {
+	 return -1;
+       }
+
+       RESET_TIMER(ti,oti);
+       if ( rd <= 0 ) {
+	 gettimeofday(&end, NULL);
+	 dt  = (end.tv_sec  - beg.tv_sec) * 1000;
+	 dt += (end.tv_usec - beg.tv_usec) / 1000;
+	 if ( dt > d4RdTimeout*3 ) {
+	   if ( debugD4 ) {
+	     fprintf(stderr,"Timeout at _readData(), dt = %ld ms\n", dt);
+	   }
+	   return -1;
+	   break;
+	 }
+	 continue;
+       }
+       else {
+	 total += rd;
+       }
+     }
+     
+     if ( debugD4 ) {
+       printHexValues("Recv: ",header,total);
+     }
+
+     if ( total == 6 ) {
+       toGet = (header[2] >> 8) + header[3] - 6;
+       if (debugD4) {
+	   fprintf(stderr, "toGet: %i\n", toGet);
+       }
+       total = 0;
+       gettimeofday(&beg, NULL);
+	 
+       if (toGet > len) {
+
+#ifdef DEBUG
+	 printf("Trying to read more data than fits into buffer.\n");
+	 printf("Data will be truncated from %d to %d bytes.\n", toGet, len);
+#endif
+	 toGet = len;
+       }
+
+       while ( total < toGet ) {
+	 SET_TIMER(ti,oti, d4RdTimeout);
+
+	 if (read_wrapper(desc, (char *) buf + total, toGet - total, &rd, 0) 
+	     != PRINTER_SUCCESS) {
+	   return -1;
+	 }
+
+	 RESET_TIMER(ti,oti);
+	 if ( rd <= 0 ) {
+	   gettimeofday(&end, NULL);
+	   dt  = (end.tv_sec  - beg.tv_sec) * 1000;
+	   dt += (end.tv_usec - beg.tv_usec) / 1000;
+	   if ( dt > d4RdTimeout*3 ) {
+	     if ( debugD4 ) {
+	       fprintf(stderr,"Timeout at _readData(), dt = %ld ms\n",dt);
+	     }
+	     return -1;
+	     break;
+	   }
+	   continue;
+         } else {
+	   total += rd;
          }
-         else
-         {
-            total += rd;
-         }
-      }
-      if ( debugD4 )
+       }
+       if ( debugD4 ) {
          printHexValues("Recv: ",buf,total);
-      return total;
-   }
+       }
+       return total;
+     }
 
-   return -1;
+     return -1;
+   }
 }
 
 /*******************************************************************/
@@ -679,15 +769,16 @@ static int _readData(int fd, unsigned char *buf, int len)
 /*                                                                 */
 /*******************************************************************/
 
-static int sendReceiveCmd(int fd, unsigned char *cmd, int len, unsigned char *answer, int expectedlen)
+static int sendReceiveCmd(printer_descriptor *desc, unsigned char *cmd, 
+			  int len, unsigned char *answer, int expectedlen)
 {
    int rd;
-   if ( (rd = writeCmd(fd, cmd, len ) ) != len )
+   if ( (rd = writeCmd(desc, cmd, len ) ) != len )
    {
       if ( rd < 0 ) return -1;
       return 0;
    }
-   rd = readAnswer(fd, answer, expectedlen );
+   rd = readAnswer(desc, answer, expectedlen );
    if ( rd == 0 )
    {
       /* no answer from device */
@@ -732,7 +823,7 @@ static int sendReceiveCmd(int fd, unsigned char *cmd, int len, unsigned char *an
 /*                                                                 */
 /*******************************************************************/
 
-int EnterIEEE(int fd)
+int EnterIEEE(printer_descriptor *desc)
 {
    unsigned char buf[200];
    unsigned char cmd[] = 
@@ -744,11 +835,11 @@ int EnterIEEE(int fd)
    int rd;
    memset(buf, 0, sizeof(buf));
 Loop:
-   if ( writeCmd(fd, cmd, sizeof(cmd) ) != sizeof(cmd) )
+   if ( writeCmd(desc, cmd, sizeof(cmd) ) != sizeof(cmd) )
    {
       return 0;
    }
-   rd = readAnswer(fd, buf, 8);
+   rd = readAnswer(desc, buf, 8);
    if ( rd == 0 )
    {
       /* no answer from device */
@@ -780,7 +871,7 @@ Loop:
 /*                                                                 */
 /*******************************************************************/
 
-int Init(int fd)
+int Init(printer_descriptor *desc)
 {
    unsigned char buf[20];
    init_t cmd;
@@ -795,7 +886,7 @@ int Init(int fd)
    cmd.head.command  = 0;
    cmd.revision      = 0x10;
     
-   rd = sendReceiveCmd(fd, (unsigned char*)&cmd, sizeof(cmd), buf, 9 );
+   rd = sendReceiveCmd(desc, (unsigned char*)&cmd, sizeof(cmd), buf, 9 );
    return rd == 9 ? 1 : 0;
 }
 
@@ -808,7 +899,7 @@ int Init(int fd)
 /*                                                                 */
 /*******************************************************************/
 
-int Exit(int fd)
+int Exit(printer_descriptor *desc)
 {
    int rd;
    unsigned char  buf[20];
@@ -821,7 +912,7 @@ int Exit(int fd)
    cmd.control  = 0;
    cmd.command  = 8;
 
-   rd = sendReceiveCmd(fd, (unsigned char*)&cmd, sizeof(cmd), buf, 8 );
+   rd = sendReceiveCmd(desc, (unsigned char*)&cmd, sizeof(cmd), buf, 8 );
    return rd > 0 ? 1 : rd;
 }
 
@@ -835,7 +926,7 @@ int Exit(int fd)
 /*                                                                 */
 /*******************************************************************/
 
-int GetSocketID(int fd, const char *serviceName)
+int GetSocketID(printer_descriptor *desc, const char *serviceName)
 {
    /* the service name may not be longer as 40 bytes */
    int len = sizeof(cmdHeader_t) + strlen(serviceName);
@@ -852,7 +943,7 @@ int GetSocketID(int fd, const char *serviceName)
    cmd->command  = 0x09;
    strcpy(buf + sizeof(cmdHeader_t), serviceName);
 
-   rd = sendReceiveCmd(fd, (unsigned char*)buf, len, rBuf, len + 2);
+   rd = sendReceiveCmd(desc, (unsigned char*)buf, len, rBuf, len + 2);
    if ( rd > 0 )
    {
       return rBuf[8];
@@ -867,14 +958,15 @@ int GetSocketID(int fd, const char *serviceName)
 /* Function OpenChannel()                                          */
 /*        handle the OpenChannel command                           */
 /* Input:  int   fd    file handle                                 */
-/* I/O:    int  *sndSz The size for sendig of datas                */
-/*         int  *recSz The size for receiving of datas             */
+/* I/O:    int  *sndSz The size for sending of data                */
+/*         int  *recSz The size for receiving of data              */
 /*                                                                 */
 /* Return: -1 on error or 1 if all is OK                           */
 /*                                                                 */
 /*******************************************************************/
 
-int OpenChannel(int fd, unsigned char sockId, int *sndSz, int *rcvSz)
+int OpenChannel(printer_descriptor *desc,  unsigned char sockId, int *sndSz, 
+		int *rcvSz)
 {
    unsigned char  cmd[17];
    unsigned char  buf[20];
@@ -900,7 +992,7 @@ int OpenChannel(int fd, unsigned char sockId, int *sndSz, int *rcvSz)
       cmd[15] = 0;    /* initial credit for us ? */
       cmd[16] = 0;
 
-      rd = sendReceiveCmd(fd, cmd, 17, buf, 16);
+      rd = sendReceiveCmd(desc, cmd, 17, buf, 16);
       if ( rd == -1 )
       {
          return -1;
@@ -909,7 +1001,7 @@ int OpenChannel(int fd, unsigned char sockId, int *sndSz, int *rcvSz)
       {
          if ( buf[7] == 4 )
          {
-            /* device cant allocate resources now */
+            /* device can't allocate resources now */
             continue;
          }
          else if ( buf[7] != 0 )
@@ -934,13 +1026,13 @@ int OpenChannel(int fd, unsigned char sockId, int *sndSz, int *rcvSz)
 /* Function CloseChannel()                                         */
 /*        handle the CloseChannel command                          */
 /* Input:  int   fd    file handle                                 */
-/*         unsigned char    socketID  he socket to close                      */
+/*         unsigned char    socketID  the socket to close          */
 /*                                                                 */
 /* Return: -1 on error or 1 if all is OK                           */
 /*                                                                 */
 /*******************************************************************/
 
-int CloseChannel(int fd, unsigned char socketID)
+int CloseChannel(printer_descriptor *desc, unsigned char socketID)
 {
    unsigned char buf[100];
    int           rd;
@@ -955,7 +1047,7 @@ int CloseChannel(int fd, unsigned char socketID)
    buf[sizeof(cmdHeader_t)+0] = socketID;
    buf[sizeof(cmdHeader_t)+1] = socketID;
    buf[sizeof(cmdHeader_t)+2] = 0;
-   rd = sendReceiveCmd(fd, buf,10, buf, 10);
+   rd = sendReceiveCmd(desc, buf,10, buf, 10);
    return rd == 10 ? 1 : rd;
 }
 
@@ -963,13 +1055,13 @@ int CloseChannel(int fd, unsigned char socketID)
 /* Function CreditRequest()                                        */
 /*        handle the CreditRequest command                         */
 /* Input:  int   fd    file handle                                 */
-/*         unsigned char    socketID  he socket to close                      */
+/*         unsigned char    socketID  the socket to close          */
 /*                                                                 */
 /* Return: -1 on error else the credit got                         */
 /*                                                                 */
 /*******************************************************************/
 
-int CreditRequest(int fd, unsigned char socketID)
+int CreditRequest(printer_descriptor *desc, unsigned char socketID)
 {
    int           rd;
    unsigned char            buf[100];
@@ -988,7 +1080,7 @@ int CreditRequest(int fd, unsigned char socketID)
    buf[sizeof(cmdHeader_t)+3] = 0x80;
    buf[sizeof(cmdHeader_t)+4] = 0xff;
    buf[sizeof(cmdHeader_t)+5] = 0xff;
-   rd = sendReceiveCmd(fd, buf, 13, rBuf, 12);
+   rd = sendReceiveCmd(desc, buf, 13, rBuf, 12);
    if ( rd == 12 )
    {
       /* this is the credit */
@@ -1004,14 +1096,14 @@ int CreditRequest(int fd, unsigned char socketID)
 /* Function Credit()                                               */
 /*        give credit to the attached device                       */
 /* Input:  int   fd    file handle                                 */
-/*         unsigned char    socketID  the socket to close                     */
+/*         unsigned char    socketID  the socket to close          */
 /*                                                                 */
 /* Return: -1 on error or 1 if all is OK                           */
 /*                                                                 */
 /*******************************************************************/
 
 /* needed for sending of commands (channel 2) or scanning */
-int Credit(int fd, unsigned char socketID, int credit)
+int Credit(printer_descriptor *desc,  unsigned char socketID, int credit)
 {
    int rd;
    unsigned char buf[100];
@@ -1029,7 +1121,7 @@ int Credit(int fd, unsigned char socketID, int credit)
    buf[sizeof(cmdHeader_t)+1] = socketID;
    buf[sizeof(cmdHeader_t)+2] = credit >> 8;
    buf[sizeof(cmdHeader_t)+3] = credit & 0xff;
-   rd = sendReceiveCmd(fd, buf, 11, rBuf, 10);
+   rd = sendReceiveCmd(desc, buf, 11, rBuf, 10);
    if ( rd == 10 )
    {
       return 1;
@@ -1045,7 +1137,7 @@ int Credit(int fd, unsigned char socketID, int credit)
 /*        Convenience function                                     */
 /*        handle the CreditRequest command                         */
 /* Input:  int   fd    file handle                                 */
-/*         unsigned char    socketID                                          */
+/*         unsigned char    socketID                               */
 /* IN/Out  int  *sndSize  for error handling                       */
 /* IN/Out  int  *rcvSize  for error handling                       */
 /*                                                                 */
@@ -1056,15 +1148,18 @@ int Credit(int fd, unsigned char socketID, int credit)
 /*                                                                 */
 /*******************************************************************/
 #define MAX_CREDIT_REQUEST 2
-int askForCredit(int fd, unsigned char socketID, int *sndSize, int *rcvSize)
+int askForCredit(printer_descriptor *desc, unsigned char socketID, 
+		 int *sndSize, int *rcvSize)
 {
    int credit = 0;
    int count  = 0;
    
    while (credit == 0 )
    {
-      while((credit=CreditRequest(fd,socketID)) == 0  && count < MAX_CREDIT_REQUEST )
+      while((credit=CreditRequest(desc,socketID)) == 0  && 
+	    count < MAX_CREDIT_REQUEST ) {
          usleep(d4RdTimeout);
+      }
 
       if ( credit == -1 )
       {
@@ -1074,13 +1169,13 @@ int askForCredit(int fd, unsigned char socketID, int *sndSize, int *rcvSize)
          }
          credit = 0;
          /* init printer and reopen the printer channel */
-         CloseChannel(fd, socketID);
-         if ( Init(fd) )
+         CloseChannel(desc, socketID);
+         if ( Init(desc) )
          {
-            OpenChannel(fd, socketID, sndSize, rcvSize);
+            OpenChannel(desc, socketID, sndSize, rcvSize);
          }
       }
-      /* if the parent died, live this loop if credit not got */ 
+      /* if the parent died, leave this loop if credit has not been received */ 
       if ( credit == 0 && getppid() == ppid )
          return 0;
       count++;
@@ -1093,16 +1188,17 @@ int askForCredit(int fd, unsigned char socketID, int *sndSize, int *rcvSize)
 /*        Convenience function                                     */
 /*        write the data to the device                             */
 /* Input:  int   fd    file handle                                 */
-/*         unsigned char    socketID  the deetination socket                  */
-/*         unsigned char   *buf       the datas to be send                    */
-/*         int   len       how many datas are to we send           */
+/*         unsigned char    socketID  the destination socket       */
+/*         unsigned char   *buf       the data to be sent          */
+/*         int   len       how many data is to be sent             */
 /*         int   eoj       set out of band flag if eoj set         */
 /*                                                                 */
 /* Return: number of bytes written or -1;                          */
 /*                                                                 */
 /*******************************************************************/
 
-int writeData(int fd, unsigned char socketID, const unsigned char *buf, int len, int eoj)
+int writeData(printer_descriptor *desc, unsigned char socketID, 
+	      const unsigned char *buf, int len, int eoj)
 {
    unsigned char  cmd[6];
    int wr = 0;
@@ -1139,7 +1235,7 @@ int writeData(int fd, unsigned char socketID, const unsigned char *buf, int len,
    while( ret > -1 && wr != len )
    {
       SET_TIMER(ti,oti,d4WrTimeout);
-      ret = SafeWrite(fd, buffer+wr, len-wr );
+      ret = SafeWrite(desc, buffer+wr, len-wr );
       RESET_TIMER(ti,oti);
       if ( ret == -1 )
       {
@@ -1163,7 +1259,8 @@ int writeData(int fd, unsigned char socketID, const unsigned char *buf, int len,
          fprintf(stderr,"%02x ", buffer[ret]);
       fprintf(stderr,"\n      ");
       for ( ret = 0; (wr > 0) && (ret < ((wr > 20) ? 20 : wr)) ; ret++ )
-         fprintf(stderr,"%c  ", isprint(buffer[ret])&&!isspace(buffer[ret])?buffer[ret]:' ');
+         fprintf(stderr,"%c  ", 
+		 isprint(buffer[ret])&&!isspace(buffer[ret])?buffer[ret]:' ');
       fprintf(stderr,"\n");
 # if PTIME
        fprintf(stderr,"Write time %5.3f s\n",(double)dt/1000000);
@@ -1183,23 +1280,24 @@ int writeData(int fd, unsigned char socketID, const unsigned char *buf, int len,
 /*        Convenience function                                     */
 /*        give credit and read then the expected datas             */
 /* Input:  int   fd    file handle                                 */
-/*         unsigned char    socketID  the destination socket                  */
-/*         unsigned char   *buf       the datas to be send                    */
-/*         int   len       howmany datas are to we send            */
+/*         unsigned char    socketID  the destination socket       */
+/*         unsigned char   *buf       the data to be sent          */
+/*         int   len       how many data is to be sent             */
 /*                                                                 */
 /* Return: number of bytes written or -1;                          */
 /*                                                                 */
 /*******************************************************************/
 
-int readData(int fd, unsigned char socketID, unsigned char *buf, int len)
+int readData(printer_descriptor *desc, unsigned char socketID, 
+	     unsigned char *buf, int len)
 {
    int ret;
    /* give credit */
-   if ( Credit(fd, socketID, 1) == 1 )
+   if ( Credit(desc, socketID, 1) == 1 )
    {
       /* wait a little bit */
       usleep(1000);
-      ret = _readData(fd, buf, len);
+      ret = _readData(desc, buf, len);
       return ret; 
    }
    else
@@ -1213,50 +1311,30 @@ int readData(int fd, unsigned char socketID, unsigned char *buf, int len)
 /*        Convenience function                                     */
 /*        give credit and read then the expected datas             */
 /* Input:  int   fd    file handle                                 */
-/*         unsigned char    socketID  the destination socket                  */
-/*         unsigned char   *buf       the datas to be send                    */
-/*         int   len       howmany datas are to we send            */
+/*         unsigned char    socketID  the destination socket       */
+/*         unsigned char   *buf       the data to be sent          */
+/*         int   len       how many data is to be sent             */
 /*                                                                 */
 /* Return: number of bytes written or -1;                          */
 /*                                                                 */
 /*******************************************************************/
 
-void flushData(int fd, unsigned char socketID)
+void flushData(printer_descriptor *desc,  unsigned char socketID)
 {
   if (debugD4)
     fprintf(stderr, "flushData %d\n", socketID);
    /* give credit */
    if (socketID != (unsigned char) -1)
      {
-       if ( Credit(fd, socketID, 1) == 1 )
+       if ( Credit(desc, socketID, 1) == 1 )
 	 {
 	   /* wait a little bit */
 	   usleep(1000);
-	   _flushData(fd);
+	   _flushData(desc);
 	 }
      }
    else
-     _flushData(fd);
-}
-
-/*******************************************************************/
-/* Function clearSndBuf()                                          */
-/*        Convenience function                                     */
-/*                                                                 */
-/* Input:  int   fd    file handle                                 */
-/*                                                                 */
-/*                                                                 */
-/*******************************************************************/
-
-static void clearSndBuf(int fd)
-{
-   char             buf[256];
-   struct itimerval ti, oti;
-   
-   SET_TIMER(ti,oti, d4RdTimeout);
-   while ( read(fd, buf, sizeof(buf) ) > 0 )
-      SET_TIMER(ti,oti, d4RdTimeout);
-   RESET_TIMER(ti,oti);
+     _flushData(desc);
 }
 
 void setDebug(int debug)
