@@ -1,6 +1,6 @@
 /* epson.c
  *
- * (c) 2006, 2014, 2015 Markus Heinz
+ * (c) 2006, 2014, 2015, 2016 Markus Heinz
  *
  * Code taken from escputil (gutenprint 5.0.0).
  *
@@ -596,7 +596,8 @@ static void do_new_status(char *buf, int bytes) {
     CARTRIDGE_RED,
     CARTRIDGE_BLUE,
     CARTRIDGE_GLOSSOPTIMIZER,
-    CARTRIDGE_LIGHTLIGHTBLACK
+    CARTRIDGE_LIGHTLIGHTBLACK,
+    CARTRIDGE_ORANGE
   };
 
   static int color_count = sizeof(colors_new) / sizeof(int);
@@ -627,8 +628,8 @@ static void do_new_status(char *buf, int bytes) {
     CARTRIDGE_CYAN,
     CARTRIDGE_MAGENTA,
     CARTRIDGE_YELLOW,
-    CARTRIDGE_UNKNOWN,
-    CARTRIDGE_UNKNOWN,
+    CARTRIDGE_LIGHTCYAN,
+    CARTRIDGE_LIGHTMAGENTA,
     CARTRIDGE_UNKNOWN,
     CARTRIDGE_UNKNOWN,
     CARTRIDGE_UNKNOWN,
