@@ -1,6 +1,6 @@
 /* libusb-utils.c
  *
- * (c) 2014, 2015 Markus Heinz
+ * (c) 2014, 2015, 2019, 2022 Markus Heinz
  *
  * This software is licensed under the terms of the GPL.
  * For details see file COPYING.
@@ -22,10 +22,10 @@
  *
  * Returns USB_SUCCESS or USB_FAILURE.
  */
-int init_usb(libusb_context *ctx) {
+int init_usb() {
   int result;
 
-  result = libusb_init(&ctx); //initialize the library for the session
+  result = libusb_init(NULL); //initialize the library for the session
   
   if (result < LIBUSB_SUCCESS) {
     
@@ -41,12 +41,9 @@ int init_usb(libusb_context *ctx) {
 #ifdef DEBUG
 
 #if (HOST_OS == LINUX)
-
-  libusb_set_debug(ctx, 4); //set verbosity level to 4
-
-#elif (HOST_OS == FREEBSD)
-
-  libusb_set_debug(ctx, 1); //set verbosity level to 1
+  
+  //set verbosity level to 4
+  libusb_set_option(NULL, LIBUSB_OPTION_LOG_LEVEL, LIBUSB_LOG_LEVEL_DEBUG);
 
 #endif
 
@@ -58,15 +55,15 @@ int init_usb(libusb_context *ctx) {
 /*
  * Deinitializes libusb.
  */
-void shutdown_usb(libusb_context *ctx) {
-  libusb_exit(ctx); //needs to be called to end the session
+void shutdown_usb() {
+  libusb_exit(NULL); //needs to be called to end the session
 }
 
 /*
  * Finds an USB printer and returns a struct describing it. If no printer is 
  * found NULL is returned.
  */
-usb_printer *find_printer(libusb_context *ctx, int instance) {
+usb_printer *find_printer(int instance) {
   libusb_device **devs; //pointer to pointer of device, 
                         //used to retrieve a list of devices
   libusb_device_handle *dev_handle; //a device handle
@@ -75,7 +72,7 @@ usb_printer *find_printer(libusb_context *ctx, int instance) {
   int printers_found = 0; // used for finding the correct printer
   usb_printer *printer = NULL;
 
-  cnt = libusb_get_device_list(ctx, &devs); //get the list of devices
+  cnt = libusb_get_device_list(NULL, &devs); //get the list of devices
 
   if (cnt < LIBUSB_SUCCESS) {
 
@@ -443,7 +440,8 @@ usb_printer *check_for_printer(libusb_device_handle *handle) {
  */
 int open_device_handle(usb_printer *printer) {
   int r;
-
+  int currentConfig;
+  
   if (printer != NULL && printer->device != NULL) {
     if (libusb_open(printer->device, &printer->handle) == LIBUSB_SUCCESS) {
 
@@ -492,18 +490,40 @@ int open_device_handle(usb_printer *printer) {
 
       assert(printer->handle != NULL);
 
-      r = libusb_set_configuration(printer->handle, printer->configuration);
+      r = libusb_get_configuration(printer->handle, &currentConfig);
 
       if (r < LIBUSB_SUCCESS) {
 
 #ifdef DEBUG
 
-	printf("Cannot set configuration: error %d\n", r);
+        printf("Cannot get configuration: error %d\n", r);
+        
+#endif
+        
+        return USB_FAILURE;
+      } else if (currentConfig == printer->configuration) {
+
+#ifdef DEBUG
+
+        printf("Configuration is already correctly\n");
 
 #endif
 
-      }
+      } else {
+      
+        r = libusb_set_configuration(printer->handle, printer->configuration);
 
+        if (r < LIBUSB_SUCCESS) {
+
+#ifdef DEBUG
+
+          printf("Cannot set configuration: error %d\n", r);
+
+#endif
+
+        }
+      }
+      
 #ifdef DEBUG
 
       printf("Trying to claim interface %d\n", printer->interface);
