@@ -1,6 +1,6 @@
 /* printer-io.c
  *
- * (c) 2014, 2015, 2018 Markus Heinz
+ * (c) 2014, 2015, 2018, 2022 Markus Heinz
  *
  * This software is licensed under the terms of the GPL.
  * For details see file COPYING.
@@ -36,13 +36,10 @@ int open_wrapper(printer_descriptor *desc) {
   }
 
   if (desc->mode == PRINTER_MODE_LIBUSB) {
-    libusb_context *ctx = NULL;
-
-    if (init_usb(ctx) == USB_SUCCESS) {
-      usb_printer *printer = find_printer(ctx, desc->portnumber);
+    if (init_usb() == USB_SUCCESS) {
+      usb_printer *printer = find_printer(desc->portnumber);
 
       if (printer != NULL) {
-	desc->ctx = ctx;
 	desc->printer = printer;
 
 	if (open_device_handle(printer) == USB_SUCCESS) {
@@ -71,7 +68,7 @@ int close_wrapper(printer_descriptor *desc) {
       
       int result = release_device_handle(desc->printer);
 
-      shutdown_usb(desc->ctx);
+      shutdown_usb();
 
       if (result == USB_SUCCESS) {
 	return PRINTER_SUCCESS;
@@ -85,7 +82,7 @@ int close_wrapper(printer_descriptor *desc) {
       printf("printer handle is NULL\n");
 #endif
       
-      shutdown_usb(desc->ctx);
+      shutdown_usb();
       return PRINTER_FAILURE;
     }
   }
